
// TreeIterator examples
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.

import com.myarch.treeiterator.TreeIterator;
import com.myarch.treeiterator.DOMAdapter;

import org.w3c.dom.Node;
import org.w3c.dom.Element;

/**
 * Demonstrates the selecive traversal using nested loops.
 * This is the least efficient way of selective traversal.
 * Use <code>skipChildren</code> or <code>setLevelDepth</code> instead.
 *
 * @author Alexander Ananiev
 */
public class SelectiveNestedTraversal {

  /**
   * Selectively traverses the tree using nested loops.
   * Traverses only the branch of "K.Tamura" and ignores other branches.
   *
   * @param rootNode root of the DOM tree.
   */
  public void traverse ( Node rootNode ) {

    // create 1st iterator. depth should be limited to ID's level
    DOMAdapter domAdapter = new DOMAdapter();
    TreeIterator tIterator = new TreeIterator ( rootNode, domAdapter, 2 );
    Node node = null;
    while( ( node=(Node)tIterator.next() )!=null ){

      // print node information
      System.out.println( node.getNodeName()+"="+node.getNodeValue());

      if  (node instanceof Element && ((Element)node).getAttribute( "id" ).
        equalsIgnoreCase( "K.TAMURA" )) {

        // create 2nd iterator, perform nested loop to process the branch.
        TreeIterator branchIterator =  new TreeIterator ( node, domAdapter);
        Node branchNode = null;
        while( (branchNode=(Node)branchIterator.next()) != null )
          // print node information
          System.out.println( branchNode.getNodeName()+"="+branchNode.getNodeValue());
      }
    }
  }

  public static void main(String[] args) {
    try {
      // default file name
      String filename = "personnel.xml";
      if (args.length == 1)
        // any file can be specified
        filename = args[0];
      new SelectiveNestedTraversal().traverse( ParserUtil.parse( filename ) );
    }
    catch( Exception ex ) {
      ex.printStackTrace();
    }
  }



}
