// TreeIterator utility
// Copyright (c) 2001 Alexander Ananiev & MyArch.com.  All rights reserved.


package com.myarch.treeiterator;

import java.util.*;
/**
 * Object pool that can be used for objects that are created for every
 * level (such as enumeration adapter).
 * Objects are allocated/released similar to the stack.
 * @author Alexander Ananiev
 */

final class ObjectPool {

  private LinkedList pool = new LinkedList();
  private ListIterator iter;
  private Factory objFactory;

  /**
   * Creates pool with the factory.
   * Factory is called every time when object needs to be created.
   */
  public ObjectPool (Factory objFactory) {
    iter=pool.listIterator();
    this.objFactory = objFactory;
  }
  /**
   * Returns object from the pool if it is available,
   * otherwise creates the new object and adds it to the pool
   */
   public Object allocate() {
    Object obj = null;

    if ( iter.hasNext() ) {
      obj = iter.next();
    }
    else {
      obj= objFactory.create();
      add( obj  );
    }
    return obj;
  }

  /**
   * Returns last used object to the pool.
   */
  public void release() {
    iter.previous();
  }

  /**
   * Adds object to the pool.
   */
  public void add( Object obj) {
    iter.add( obj );
  }

  /**
   * Interface used to create objects.
   */
  interface Factory {
    public Object create();
  }
}

